@@ -1,11 +1,11 @@
 /************************************************************
- * HMMER - Biological sequence analysis with profile-HMMs
- * Copyright (C) 1992-1998 Sean R. Eddy
- *
- *   This source code is distributed under the terms of the
- *   GNU General Public License. See the files COPYING and
- *   GNULICENSE for details.
- *
+ * HMMER - Biological sequence analysis with profile HMMs
+ * Copyright (C) 1992-1998 Washington University School of Medicine
+ * All Rights Reserved
+ * 
+ *     This source code is distributed under the terms of the
+ *     GNU General Public License. See the files COPYING and LICENSE
+ *     for details.
  ************************************************************/
 
 /* histogram.c
@@ -14,7 +14,7 @@
  * Accumulation, printing, and fitting of score histograms
  * from database searches.
  *
- * RCS $Id: histogram.c,v 1.11 1998/01/19 12:21:38 eddy Exp $
+ * RCS $Id: histogram.c,v 1.15 1998/10/09 17:40:25 eddy Exp $
  ************************************************************
  * Basic API:
  * 
@@ -32,6 +32,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
+#include <float.h>
 #include <math.h>
 
 #include "squid.h"
@@ -39,7 +40,9 @@
 #include "structs.h"
 #include "funcs.h"
 
-#include <assert.h>
+#ifdef MEMDEBUG
+#include "dbmalloc.h"
+#endif
 
 /* Function: AllocHistogram()
  * 
@@ -77,8 +80,6 @@ AllocHistogram(int min, int max, int lumpsize)
   h->expect    = NULL;
   h->fit_type  = HISTFIT_NONE;
 
-  h->param[EVD_WONKA] = 1.0;	/* just in case, make sure this initializes */
-
   return h;
 }
 
@@ -170,10 +171,8 @@ AddToHistogram(struct histogram_s *h, float sc)
   if (score < h->lowscore) h->lowscore   = score;
   if (score > h->highscore) h->highscore = score;
 
-#if DEBUGLEVEL >= DEBUG_AGGRESSIVE
-  fprintf(stderr, "AddToHistogram(): added %.1f; rounded to %d; in bin %d (%d-%d)\n",
-	  sc, score, score-h->min, h->min, h->max);
-#endif
+  SQD_DPRINTF3(("AddToHistogram(): added %.1f; rounded to %d; in bin %d (%d-%d)\n",
+	  sc, score, score-h->min, h->min, h->max));
   return;
 }
 
@@ -302,16 +301,6 @@ PrintASCIIHistogram(FILE *fp, struct histogram_s *h)
        */
       if (h->fit_type != HISTFIT_NONE && (int) h->expect[idx] > 0)
 	{
-				/* "corrected" line */
-#ifdef SRE_REMOVED
-	  if (h->fit_type == HISTFIT_EVD)
-	    {
-	      pos = 20 + (int)(h->param[EVD_WONKA] * h->expect[idx] - 1) / units;
-	      if (pos >= 78) pos = 78; /* be careful of buffer bounds */
-	      buffer[pos] = 'o';
-	    }
-#endif
-				/* true (uncorrected) line */
 	  pos = 20 + (int)(h->expect[idx]-1) / units;
 	  if (pos >= 78) pos = 78; /* be careful of buffer bounds */
 	  buffer[pos] = '*';
@@ -333,9 +322,6 @@ PrintASCIIHistogram(FILE *fp, struct histogram_s *h)
     fprintf(fp, "\n\n%% Statistical details of theoretical EVD fit:\n");
     fprintf(fp, "              mu = %10.4f\n", h->param[EVD_MU]);
     fprintf(fp, "          lambda = %10.4f\n", h->param[EVD_LAMBDA]);
-#ifdef SRE_REMOVED
-    fprintf(fp, "    fraction fit = %10.4f\n", h->param[EVD_WONKA]);
-#endif
     fprintf(fp, "chi-sq statistic = %10.4f\n", h->chisq);
     fprintf(fp, "  P(chi-square)  = %10.4g\n", h->chip);
     break;
@@ -539,7 +525,7 @@ EVDBasicFit(struct histogram_s *h)
   /* Set the EVD parameters in the histogram;
    * pass 2 for additional lost degrees of freedom because we fit mu, lambda.
    */
-  ExtremeValueSetHistogram(h, mu, lambda, h->lowscore, h->highscore, 1.0, 2);
+  ExtremeValueSetHistogram(h, mu, lambda, h->lowscore, h->highscore, 2);
 
   free(x);
   free(d);
@@ -665,15 +651,11 @@ ExtremeValueFitHistogram(struct histogram_s *h, int censor, float high_hint)
     }
 
   /* Set the histogram parameters;
-   * - the wonka factor is n+z / h->total : e.g. that's the fraction of the
-   *   hits that we expect to match the EVD, others are generally lower
    * - we fit from lowbound to highbound; thus we lose 2 degrees of freedom
    *   for fitting mu, lambda, but we get 1 back because we're unnormalized
    *   in this interval, hence we pass 2-1 = 1 as ndegrees.
-   *   
-   *   Mon Jan 19 06:18:14 1998: wonka = 1.0, temporarily disabled.
    */    
-  ExtremeValueSetHistogram(h, mu, lambda, lowbound, highbound, 1.0, 1); 
+  ExtremeValueSetHistogram(h, mu, lambda, lowbound, highbound, 1); 
   return 1;
 
 FITFAILED:
@@ -689,23 +671,18 @@ FITFAILED:
  * Purpose:  Instead of fitting the histogram to an EVD,
  *           simply set the EVD parameters from an external source.
  *
- *           Note that the fudge factor "wonka" is used /only/
- *           for prettification of expected/theoretical curves
- *           in PrintASCIIHistogram displays.
- *           
  * Args:     h        - the histogram to set
  *           mu       - mu location parameter                
  *           lambda   - lambda scale parameter
  *           lowbound - low bound of the histogram that was fit
  *           highbound- high bound of histogram that was fit
- *           wonka    - fudge factor; fraction of hits estimated to be "EVD-like"
  *           ndegrees - extra degrees of freedom to subtract in X^2 test:
  *                        typically 0 if mu, lambda are parametric,
  *                        else 2 if mu, lambda are estimated from data
  */
 void
 ExtremeValueSetHistogram(struct histogram_s *h, float mu, float lambda, 
-			 float lowbound, float highbound, float wonka, int ndegrees)
+			 float lowbound, float highbound, int ndegrees)
 {
   int   sc;
   int   hsize, idx;
@@ -716,7 +693,6 @@ ExtremeValueSetHistogram(struct histogram_s *h, float mu, float lambda,
   h->fit_type          = HISTFIT_EVD;
   h->param[EVD_LAMBDA] = lambda;
   h->param[EVD_MU]     = mu;
-  h->param[EVD_WONKA]  = wonka;
 
   hsize     = h->max - h->min + 1;
   h->expect = (float *) MallocOrDie(sizeof(float) * hsize);
@@ -950,6 +926,10 @@ double
 ExtremeValueP(float x, float mu, float lambda)
 {
   double y;
+			/* avoid overflow fp exceptions */
+  if ((lambda * (x - mu)) < -1. * log(log(DBL_MAX))) return 1.0;
+			/* avoid underflow fp exceptions */
+  if ((lambda * (x - mu)) > log(DBL_MAX)) return 0.0;
 			/* a roundoff issue arises; use 1 - e^-x --> x for small x */
   y = exp(-1. * lambda * (x - mu));
   if (y < 1e-7) return y;
@@ -1196,7 +1176,7 @@ EVDMaxLikelyFit(float *x, int *c, int n, float *ret_mu, float *ret_lambda)
   if (i == 100)
     {
       float left, right, mid;
-      Worry(DEBUG_LIGHT, "Newton/Raphson failed; switchover to bisection");
+      SQD_DPRINTF2(("EVDMaxLikelyFit(): Newton/Raphson failed; switchover to bisection"));
 
 				/* First we need to bracket the root */
       lambda = right = left = 0.2;
@@ -1206,7 +1186,7 @@ EVDMaxLikelyFit(float *x, int *c, int n, float *ret_mu, float *ret_lambda)
 	  do {
 	    left -= 0.1;
 	    if (left < 0.) { 
-	      Worry(DEBUG_LIGHT, "failed to bracket root"); 
+	      SQD_DPRINTF2(("EVDMaxLikelyFit(): failed to bracket root")); 
 	      return 0; 
 	    }
 	    Lawless416(x, c, n, left, &fx, &dfx);
@@ -1218,7 +1198,7 @@ EVDMaxLikelyFit(float *x, int *c, int n, float *ret_mu, float *ret_lambda)
 	    right += 0.1;
 	    Lawless416(x, c, n, right, &fx, &dfx);
 	    if (right > 100.) {
-	      Worry(DEBUG_LIGHT, "failed to bracket root"); 
+	      SQD_DPRINTF2(("EVDMaxLikelyFit(): failed to bracket root")); 
 	      return 0; 
 	    }
 	  } while (fx > 0.);
@@ -1233,7 +1213,7 @@ EVDMaxLikelyFit(float *x, int *c, int n, float *ret_mu, float *ret_lambda)
 	  else          right = mid;
 	}
       if (i == 100) { 
-	Worry(DEBUG_LIGHT, "even the bisection search failed"); 
+	SQD_DPRINTF2(("EVDMaxLikelyFit(): even the bisection search failed")); 
 	return 0; 
       }
       lambda = mid;
@@ -1242,6 +1222,7 @@ EVDMaxLikelyFit(float *x, int *c, int n, float *ret_mu, float *ret_lambda)
   /* 3. Substitute into Lawless 4.1.5 to find mu
    */
   esum = 0.;
+  total = 0.;
   for (i = 0; i < n; i++)
     {
       mult   = (c == NULL) ? 1. : (double) c[i];
@@ -1321,7 +1302,7 @@ EVDCensoredFit(float *x, int *y, int n, int z, float c,
     {
       float left, right, mid;
 				/* First we need to bracket the root */
-      Worry(DEBUG_LIGHT, "Newton/Raphson failed; switched to bisection");
+      SQD_DPRINTF2(("EVDCensoredFit(): Newton/Raphson failed; switched to bisection"));
       lambda = right = left = 0.2;
       Lawless422(x, y, n, z, c, lambda, &fx, &dfx);
       if (fx < 0.) 
@@ -1329,7 +1310,7 @@ EVDCensoredFit(float *x, int *y, int n, int z, float c,
 	  do {
 	    left -= 0.03;
 	    if (left < 0.) { 
-	      Worry(DEBUG_LIGHT, "failed to bracket root"); 
+	      SQD_DPRINTF2(("EVDCensoredFit(): failed to bracket root")); 
 	      return 0;
 	    }
 	    Lawless422(x, y, n, z, c, left, &fx, &dfx);
@@ -1341,7 +1322,7 @@ EVDCensoredFit(float *x, int *y, int n, int z, float c,
 	    right += 0.1;
 	    Lawless422(x, y, n, z, c, left, &fx, &dfx);
 	    if (right > 100.) {
-	      Worry(DEBUG_LIGHT, "failed to bracket root");
+	      SQD_DPRINTF2(("EVDCensoredFit(): failed to bracket root"));
 	      return 0;
 	    }
 	  } while (fx > 0.);
@@ -1356,7 +1337,7 @@ EVDCensoredFit(float *x, int *y, int n, int z, float c,
 	  else          right = mid;
 	}
       if (i == 100) {
-	Worry(DEBUG_LIGHT, "even the bisection search failed");
+	SQD_DPRINTF2(("EVDCensoredFit(): even the bisection search failed"));
 	return 0;
       }
       lambda = mid;
