@@ -1,18 +1,18 @@
 /************************************************************
- * HMMER - Biological sequence analysis with profile-HMMs
- * Copyright (C) 1992-1998 Sean R. Eddy
- *
- *   This source code is distributed under the terms of the
- *   GNU General Public License. See the files COPYING and
- *   GNULICENSE for details.
- *
+ * HMMER - Biological sequence analysis with profile HMMs
+ * Copyright (C) 1992-1998 Washington University School of Medicine
+ * All Rights Reserved
+ * 
+ *     This source code is distributed under the terms of the
+ *     GNU General Public License. See the files COPYING and LICENSE
+ *     for details.
  ************************************************************/
 
 /* hmmbuild.c
  * SRE, Mon Nov 18 12:41:29 1996
  *
  * main() for HMM construction from an alignment.
- * RCS $Id: hmmbuild.c,v 1.14 1998/05/27 02:44:32 eddy Exp $
+ * RCS $Id: hmmbuild.c,v 1.17 1998/12/09 00:43:16 eddy Exp $
  */
 
 #include <stdio.h>
@@ -34,16 +34,16 @@ static char banner[] = "hmmbuild - build a hidden Markov model from an alignment
 static char usage[]  = "\
 Usage: hmmbuild [-options] <hmmfile output> <alignment file>\n\
   Available options are:\n\
-   -h        : help; print brief help on version and usage\n\
-   -n <s>    : name; name this HMM <s>\n\
-   -o <file> : re-save annotated alignment to <file>\n\
-   -A        : append; append this HMM to <hmmfile>\n\
-   -F        : force; allow overwriting of <hmmfile>\n\
+   -h     : help; print brief help on version and usage\n\
+   -n <s> : name; name this HMM <s>\n\
+   -o <f> : re-save annotated alignment to <f>\n\
+   -A     : append; append this HMM to <hmmfile>\n\
+   -F     : force; allow overwriting of <hmmfile>\n\
 \n\
   Alternative search algorithm styles: (default: hmmls domain alignment)\n\
-   -f        : multi-hit local (hmmfs style)\n\
-   -g        : global alignment (hmms style, Needleman/Wunsch)\n\
-   -s        : local alignment (hmmsw style, Smith/Waterman)\n\
+   -f     : multi-hit local (hmmfs style)\n\
+   -g     : global alignment (hmms style, Needleman/Wunsch)\n\
+   -s     : local alignment (hmmsw style, Smith/Waterman)\n\
 ";
 
 static char experts[] = "\
@@ -52,9 +52,9 @@ static char experts[] = "\
    --hand    : manual construction (requires SELEX file, #=RF annotation)\n\
 \n\
   Expert customization of parameters and priors:\n\
-   --null  <file> : read null (random sequence) model from <file>\n\
-   --pam <file>   : heuristic PAM-based prior, using BLAST PAM matrix in <file>\n\
-   --prior <file> : read Dirichlet prior parameters from <file>\n\
+   --null  <f> : read null (random sequence) model from <f>\n\
+   --pam   <f> : heuristic PAM-based prior, using BLAST PAM matrix in <f>\n\
+   --prior <f> : read Dirichlet prior parameters from <f>\n\
 \n\
   Alternative sequence weighting strategies: (default: GSC weights)\n\
    --wblosum     : Henikoff simple filter weights (see --idlevel)\n\
@@ -71,7 +71,7 @@ static char experts[] = "\
   Other expert options:\n\
    --archpri <x> : set architecture size prior to <x> {0.85} [0..1]\n\
    --binary      : save the model in binary format, not ASCII text\n\
-   --cfile <file>: save count vectors to <file>\n\
+   --cfile <f>   : save count vectors to <f>\n\
    --gapmax <x>  : max fraction of gaps in mat column {0.50} [0..1]\n\
    --idlevel <x> : set frac. id level used by eff. nseq and --wblosum {0.62}\n\
    --pamwgt <x>  : set weight on PAM-based prior to <x> {20.}[>=0]\n\
@@ -212,7 +212,6 @@ main(int argc, char **argv)
     else if (strcmp(optname, "-g") == 0) cfg_strategy      = P7_BASE_CONFIG;
     else if (strcmp(optname, "-n") == 0) name              = Strdup(optarg); 
     else if (strcmp(optname, "-o") == 0) align_ofile       = optarg;
-    else if (strcmp(optname, "-r") == 0) rndfile           = optarg;
     else if (strcmp(optname, "-s") == 0) cfg_strategy      = P7_SW_CONFIG;
     if      (strcmp(optname, "-A") == 0) do_append         = TRUE; 
     else if (strcmp(optname, "-F") == 0) overwrite_protect = FALSE;
@@ -221,11 +220,12 @@ main(int argc, char **argv)
     else if (strcmp(optname, "--binary")  == 0) do_binary     = TRUE;
     else if (strcmp(optname, "--cfile")   == 0) cfile         = optarg;
     else if (strcmp(optname, "--fast")    == 0) c_strategy    = P7_FAST_CONSTRUCTION;
-    else if (strcmp(optname, "--hand")    == 0) c_strategy    = P7_HAND_CONSTRUCTION;
     else if (strcmp(optname, "--gapmax")  == 0) gapmax        = atof(optarg);
+    else if (strcmp(optname, "--hand")    == 0) c_strategy    = P7_HAND_CONSTRUCTION;
     else if (strcmp(optname, "--idlevel") == 0) blosumlevel   = atof(optarg);
     else if (strcmp(optname, "--noeff")   == 0) do_eff        = FALSE;
     else if (strcmp(optname, "--nucleic") == 0) SetAlphabet(hmmNUCLEIC);
+    else if (strcmp(optname, "--null")    == 0) rndfile       = optarg;
     else if (strcmp(optname, "--pam")     == 0) pamfile       = optarg;
     else if (strcmp(optname, "--pamwgt")  == 0) pamwgt        = atof(optarg);
     else if (strcmp(optname, "--prior")   == 0) prifile       = optarg;
@@ -279,7 +279,7 @@ main(int argc, char **argv)
   for (idx = 0; idx < ainfo.nseq; idx++)
     s2upper(aseq[idx]);
 				/* Set up the alphabet globals */
-  if (Alphabet_type == 0) DetermineAlphabet(aseq, ainfo.nseq);
+  if (Alphabet_type == hmmNOTSETYET) DetermineAlphabet(aseq, ainfo.nseq);
 
 				/* Set up Dirichlet priors */
   if (prifile == NULL)  pri = P7DefaultPrior();
@@ -327,6 +327,9 @@ main(int argc, char **argv)
   else if (c_strategy==P7_FAST_CONSTRUCTION) printf("Fast/ad hoc (gapmax %.2f)\n", gapmax);
   else                                       printf("MAP (gapmax hint: %.2f)\n", gapmax);
 
+  printf("Null model used:                   %s\n",
+	 (rndfile == NULL) ? "(default)" : rndfile);
+
   printf("Prior used:                        %s\n",
 	 (prifile == NULL) ? "(default)" : prifile);
 
@@ -447,13 +450,35 @@ main(int argc, char **argv)
       printf("----------------------------------------------\n\n");
     }
 
-  /* Give the model a name; by default, the name of the alignment file
+  /* Give the model a name.
+   * If -n option is used (to override naming), use that name.
+   * Else if the alignment carries a name, use that.
+   * Else, if all that failed, use the name of the alignment file
    * without any filename extension (i.e. "globins.slx" becomes "globins"
    */
   printf("%-40s ... ", "Setting model name, etc.");
   fflush(stdout);
-  if (name == NULL) name = FileTail(seqfile, TRUE);
-  Plan7SetName(hmm, name);
+  if      (name       != NULL) Plan7SetName(hmm, name);
+  else if (ainfo.name != NULL) Plan7SetName(hmm, ainfo.name);
+  else {
+    name = FileTail(seqfile, TRUE);
+    Plan7SetName(hmm, name);
+  }
+
+  /* Transfer other information from the alignment to
+   * the HMM. This typically only works for SELEX format
+   * alignments, so these things are conditional/optional.
+   */
+  if (ainfo.acc  != NULL) Plan7SetAccession(hmm, ainfo.acc);
+  if (ainfo.desc != NULL) Plan7SetDescription(hmm, ainfo.desc);
+
+  if (ainfo.flags & AINFO_GA) { hmm->ga1 = ainfo.ga1; hmm->ga2 = ainfo.ga2; }
+  if (ainfo.flags & AINFO_TC) { hmm->tc1 = ainfo.tc1; hmm->tc2 = ainfo.tc2; }
+  if (ainfo.flags & AINFO_NC) { hmm->nc1 = ainfo.nc1; hmm->nc2 = ainfo.nc2; }
+
+  /* Record some other miscellaneous information in the HMM,
+   * like how/when we built it.
+   */
   Plan7ComlogAppend(hmm, argc, argv);
   Plan7SetCtime(hmm);
   hmm->nseq = ainfo.nseq;
