@@ -1,15 +1,17 @@
-/* SQUID - A C function library for biological sequence analysis
- * Copyright (C) 1992-1996 Sean R. Eddy	
- *
- *    This source code is distributed under terms of the
- *    GNU General Public License. See the files COPYING 
- *    and GNULICENSE for further details.
- *
- */
+/*****************************************************************
+ * HMMER - Biological sequence analysis with profile HMMs
+ * Copyright (C) 1992-1998 Washington University School of Medicine
+ * All Rights Reserved
+ * 
+ *     This source code is distributed under the terms of the
+ *     GNU General Public License. See the files COPYING and LICENSE
+ *     for details.
+ *****************************************************************/
 
 /* sqerror.c
  * 
  * error handling for the squid library
+ * RCS $Id: sqerror.c,v 1.3 1998/10/09 18:07:16 eddy Exp $
  */
 
 				/* a global errno equivalent */
@@ -68,3 +70,26 @@ Warn(char *format, ...)
   fprintf(stderr, "\n");
   fflush(stderr);
 }
+
+/* Function: Panic()
+ * 
+ * Purpose:  Die from a lethal error that's not my problem,
+ *           but instead a failure of a StdC/POSIX call that
+ *           shouldn't fail. Call perror() to get the
+ *           errno flag, then die.
+ *           
+ *           Usually called by the PANIC macro which adds
+ *           the __FILE__ and __LINE__ information; see
+ *           structs.h.
+ *           
+ *           Inspired by code in Donald Lewine's book, _POSIX 
+ *           Programmer's Guide_.
+ */
+void
+Panic(char *file, int line)
+{
+  (void) fprintf(stderr, "\nPANIC [%s line %d] ", file, line);
+  (void) perror("Unusual error");
+  exit(EXIT_FAILURE);
+}
+
