@@ -1,16 +1,16 @@
 /************************************************************
- * HMMER - Biological sequence analysis with profile-HMMs
+ * HMMER - Biological sequence analysis with profile HMMs
  * Copyright (C) 1992-1998 Washington University School of Medicine
- *
- *   This source code is distributed under the terms of the
- *   GNU General Public License. See the files COPYING and
- *   GNULICENSE for details.
- *
+ * All Rights Reserved
+ * 
+ *     This source code is distributed under the terms of the
+ *     GNU General Public License. See the files COPYING and LICENSE
+ *     for details.
  ************************************************************/
 
 /* trace.c
  * SRE, Sat Nov 16 12:34:57 1996
- * RCS $Id: trace.c,v 1.7 1998/03/11 17:12:04 eddy Exp $
+ * RCS $Id: trace.c,v 1.13 1998/12/09 17:41:15 eddy Exp $
  * 
  * Support for Plan 7 traceback data structure, p7trace_s.
  */
@@ -40,18 +40,18 @@ P7AllocTrace(int tlen, struct p7trace_s **ret_tr)
 {
   struct p7trace_s *tr;
   
-  tr = (struct p7trace_s *) MallocOrDie (sizeof(struct p7trace_s));
-  tr->statetype = (enum p7stype *)  MallocOrDie (sizeof(enum p7stype) * tlen);
-  tr->nodeidx   = (int *)   MallocOrDie (sizeof(int)  * tlen);
-  tr->pos       = (int *)   MallocOrDie (sizeof(int)  * tlen);
+  tr =            MallocOrDie (sizeof(struct p7trace_s));
+  tr->statetype = MallocOrDie (sizeof(char) * tlen);
+  tr->nodeidx   = MallocOrDie (sizeof(int)  * tlen);
+  tr->pos       = MallocOrDie (sizeof(int)  * tlen);
   *ret_tr = tr;
 }
 void
 P7ReallocTrace(struct p7trace_s *tr, int tlen)
 {
-  tr->statetype = (enum p7stype *) ReallocOrDie (tr->statetype, tlen * sizeof(enum p7stype));
-  tr->nodeidx   = (int *)  ReallocOrDie (tr->nodeidx,   tlen * sizeof(int));
-  tr->pos       = (int *)  ReallocOrDie (tr->pos,       tlen * sizeof(int));
+  tr->statetype = ReallocOrDie (tr->statetype, tlen * sizeof(char));
+  tr->nodeidx   = ReallocOrDie (tr->nodeidx,   tlen * sizeof(int));
+  tr->pos       = ReallocOrDie (tr->pos,       tlen * sizeof(int));
 }
 void 
 P7FreeTrace(struct p7trace_s *tr)
@@ -78,7 +78,7 @@ P7FreeTrace(struct p7trace_s *tr)
  * Returns:  void
  */
 void
-TraceSet(struct p7trace_s *tr, int tpos, enum p7stype type, int idx, int pos)
+TraceSet(struct p7trace_s *tr, int tpos, char type, int idx, int pos)
 {
   tr->statetype[tpos] = type;
   tr->nodeidx[tpos]   = idx;
@@ -86,6 +86,39 @@ TraceSet(struct p7trace_s *tr, int tpos, enum p7stype type, int idx, int pos)
 }
 
 
+/* Function: MergeTraceArrays()
+ * Date:     SRE, Sun Jul  5 15:09:10 1998 [St. Louis]
+ *
+ * Purpose:  Combine two arrays of traces into a single array.
+ *           Used in hmmalign to merge traces from a fixed alignment
+ *           with traces from individual unaligned seqs.
+ * 
+ *           t1 traces always precede t2 traces in the resulting array.
+ *
+ * Args:     t1 - first set of traces
+ *           n1 - number of traces in t1
+ *           t2 - second set of traces
+ *           n2 - number of traces in t2
+ *
+ * Returns:  pointer to new array of traces.
+ *           Both t1 and t2 are free'd here! Do not reuse.
+ */
+struct p7trace_s **
+MergeTraceArrays(struct p7trace_s **t1, int n1, struct p7trace_s **t2, int n2)
+{
+  struct p7trace_s **tr;
+  int i;			/* index in traces */
+
+  tr = MallocOrDie(sizeof(struct p7trace_s *) * (n1+n2));
+  for (i = 0; i < n1; i++) tr[i]    = t1[i];
+  for (i = 0; i < n2; i++) tr[n1+i] = t2[i];
+  free(t1);
+  free(t2);
+  return tr;
+}
+
+
+
 /* Function: P7ReverseTrace()
  * Date:     SRE, Mon Aug 25 12:57:29 1997; Denver CO. 
  * 
@@ -108,16 +141,16 @@ TraceSet(struct p7trace_s *tr, int tpos, enum p7stype type, int idx, int pos)
 void
 P7ReverseTrace(struct p7trace_s *tr)
 {
-  enum p7stype *statetype;
-  int          *nodeidx;
-  int          *pos;
-  int           opos, npos;
+  char  *statetype;
+  int   *nodeidx;
+  int   *pos;
+  int    opos, npos;
 
   /* Allocate
    */
-  statetype = (enum p7stype *) MallocOrDie (sizeof(enum p7stype) * tr->tlen);
-  nodeidx   = (int *) MallocOrDie (sizeof(int) * tr->tlen);
-  pos       = (int *) MallocOrDie (sizeof(int) * tr->tlen);
+  statetype = MallocOrDie (sizeof(char)* tr->tlen);
+  nodeidx   = MallocOrDie (sizeof(int) * tr->tlen);
+  pos       = MallocOrDie (sizeof(int) * tr->tlen);
   
   /* Reverse the trace.
    */
@@ -321,7 +354,8 @@ P7TraceScore(struct plan7_s *hmm, char *dsq, struct p7trace_s *tr)
  *           are split in half and the halves are justified in
  *           each direction (the objective being to increase
  *           the chances of getting insertions aligned well enough
- *           for them to become a match).
+ *           for them to become a match). SAM gap char conventions
+ *           are used: - in match columns, . in insert columns
  * 
  * NOTE:     Does not recognize J state.
  *
@@ -429,6 +463,8 @@ P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, int nseq, int mlen,
 				/* blank an aseq */
     for (apos = 0; apos < alen; apos++)
       aseqs[idx][apos] = '.';
+    for (k = 1; k <= mlen; k++)
+      aseqs[idx][matmap[k]] = '-';
     aseqs[idx][alen] = '\0';
 				/* align the sequence */
     apos = 0;
@@ -484,8 +520,8 @@ P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, int nseq, int mlen,
    * Build ainfo
    ***********************************************/
         
+  ainfo->au = MallocOrDie(sizeof(char) * (strlen(RELEASE)+7));
   sprintf(ainfo->au, "HMMER %s", RELEASE);
-  ainfo->flags |= AINFO_AUTH;
 				/* copy sqinfo array and weights */
   for (idx = 0; idx < nseq; idx++)
     {
@@ -495,7 +531,6 @@ P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, int nseq, int mlen,
 
   /* #=RF annotation: x for match column, . for insert column
    */
-  ainfo->flags |= AINFO_RF;
   ainfo->rf = (char *) MallocOrDie (sizeof(char) * (alen+1));
   for (apos = 0; apos < alen; apos++)
     ainfo->rf[apos] = '.';
@@ -520,8 +555,8 @@ P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, int nseq, int mlen,
  *           return the integer score for that transition. 
  */
 int
-TransitionScoreLookup(struct plan7_s *hmm, enum p7stype st1, int k1, 
-		      enum p7stype st2, int k2)
+TransitionScoreLookup(struct plan7_s *hmm, char st1, int k1, 
+		      char st2, int k2)
 {
   switch (st1) {
   case STS: return 0;	/* S never pays */
@@ -622,9 +657,9 @@ CreateFancyAli(struct p7trace_s *tr, struct plan7_s *hmm,
   ali         = AllocFancyAli();
   ali->rfline = NULL;
   ali->csline = NULL;
-  ali->model  = (char *) MallocOrDie (sizeof(char) * (tr->tlen+1));
-  ali->mline  = (char *) MallocOrDie (sizeof(char) * (tr->tlen+1));
-  ali->aseq   = (char *) MallocOrDie (sizeof(char) * (tr->tlen+1));
+  ali->model  = MallocOrDie (sizeof(char) * (tr->tlen+1));
+  ali->mline  = MallocOrDie (sizeof(char) * (tr->tlen+1));
+  ali->aseq   = MallocOrDie (sizeof(char) * (tr->tlen+1));
 
   memset(ali->model,  ' ', tr->tlen);
   memset(ali->mline,  ' ', tr->tlen);
@@ -958,6 +993,178 @@ TraceSimpleBounds(struct p7trace_s *tr, int *ret_i1, int *ret_i2,
 }
 
 
+/* Function: MasterTraceFromMap()
+ * Date:     SRE, Tue Jul  7 18:51:11 1998 [St. Louis]
+ *
+ * Purpose:  Convert an alignment map (e.g. hmm->map) to
+ *           a master trace. Used for mapping an alignment
+ *           onto an HMM. Generally precedes a call to
+ *           ImposeMasterTrace(). Compare P7ViterbiAlignAlignment(),
+ *           which aligns an alignment to the model using a
+ *           Viterbi algorithm to get a master trace. 
+ *           MasterTraceFromMap() only works if the alignment
+ *           is exactly the one used to train the model.
+ *
+ * Args:     map  - the map (usually hmm->map is passed) 1..M
+ *           M    - length of map (model; usually hmm->M passed)
+ *           alen - length of alignment that map refers to
+ *
+ * Returns:  ptr to master trace
+ *           Caller must free: P7FreeTrace().
+ */
+struct p7trace_s *
+MasterTraceFromMap(int *map, int M, int alen)
+{
+  struct p7trace_s *tr;         /* RETURN: master trace */ 
+  int tpos;			/* position in trace */
+  int apos;			/* position in alignment, 1..alen */
+  int k;			/* position in model */
+
+  /* Allocate for the trace.
+   * S-N-B- ... - E-C-T  : 6 states + alen is maximum trace,
+   * because each of alen columns is an N*, M*, I*, or C* metastate.
+   * No D* metastates possible.
+   */
+  P7AllocTrace(alen+6, &tr);
+
+  /* Initialize the trace
+   */
+  tpos = 0;
+  TraceSet(tr, tpos, STS, 0, 0); tpos++;
+  TraceSet(tr, tpos, STN, 0, 0); tpos++;
+
+  /* Leading N's
+   */
+  for (apos = 1; apos < map[1]; apos++) {
+    TraceSet(tr, tpos, STN, 0, apos); tpos++;
+  } /* now apos == map[1] */
+  TraceSet(tr, tpos, STB, 0, 0); tpos++;
+
+  for (k = 1; k < M; k++)
+    {
+      TraceSet(tr, tpos, STM, k, apos); tpos++;
+      apos++;
+
+      for (; apos < map[k+1]; apos++) {
+	TraceSet(tr, tpos, STI, k, apos); tpos++;
+      }
+    } /* now apos == map[M] and k == M*/
+      
+  TraceSet(tr, tpos, STM, M, apos); tpos++;
+  apos++;
+
+  /* Trailing C's
+   */
+  TraceSet(tr, tpos, STE, 0, 0); tpos++;
+  TraceSet(tr, tpos, STC, 0, 0); tpos++;
+  for (; apos <= alen; apos++) {
+    TraceSet(tr, tpos, STC, 0, apos); tpos++;
+  }
+
+  /* Terminate and return
+   */
+  TraceSet(tr, tpos, STT, 0, 0); tpos++;
+  tr->tlen = tpos;
+  return tr;
+}
+
+
+
+/* Function: ImposeMasterTrace()
+ * Date:     SRE, Sun Jul  5 14:27:16 1998 [St. Louis]
+ *
+ * Purpose:  Goes with P7ViterbiAlignAlignment(), which gives us
+ *           a "master trace" for a whole alignment. Now, given
+ *           the alignment and the master trace, construct individual
+ *           tracebacks for each sequence. Later we'll hand these
+ *           (and presumably other traces) to P7Traces2Alignment().
+ *           
+ *           It is possible to generate individual traces that
+ *           are not consistent with Plan7 (e.g. D->I and I->D 
+ *           transitions may be present). P7Traces2Alignment()
+ *           can handle such traces; other functions may not.
+ *           See modelmaker.c:trace_doctor() if this is a problem.
+ * 
+ *           Akin to modelmaker.c:fake_tracebacks().
+ *
+ * Args:     aseq  - aligned seqs
+ *           nseq  - number of aligned seqs 
+ *           mtr   - master traceback
+ *           ret_tr- RETURN: array of individual tracebacks, one for each aseq
+ *
+ * Returns:  (void)
+ */
+void
+ImposeMasterTrace(char **aseq, int nseq, struct p7trace_s *mtr, struct p7trace_s ***ret_tr)
+{
+  struct p7trace_s **tr;
+  int  idx;			/* counter over sequences */
+  int  i;                       /* position in raw sequence (1..L) */
+  int  tpos;			/* position in traceback           */
+  int  mpos;			/* position in master trace        */
+
+  tr = (struct p7trace_s **) MallocOrDie (sizeof(struct p7trace_s *) * nseq);
+  
+  for (idx = 0; idx < nseq; idx++)
+    {
+      P7AllocTrace(mtr->tlen, &tr[idx]); /* we're guaranteed that individuals len < master len */
+      
+      tpos = 0;
+      i    = 1;
+      for (mpos = 0; mpos < mtr->tlen; mpos++)
+	{
+	  switch (mtr->statetype[mpos]) 
+	    {
+	    case STS:		/* straight copies w/ no emission: S, B, D, E, T*/
+	    case STB:
+	    case STD:
+	    case STE:
+	    case STT:
+	      TraceSet(tr[idx], tpos, mtr->statetype[mpos], mtr->nodeidx[mpos], 0);
+	      tpos++;
+	      break;
+
+	    case STM:		/* M* implies M or D */
+	      if (isgap(aseq[idx][mtr->pos[mpos]-1])) 
+		TraceSet(tr[idx], tpos, STD, mtr->nodeidx[mpos], 0);
+	      else {
+		TraceSet(tr[idx], tpos, STM, mtr->nodeidx[mpos], i);
+		i++;
+	      }
+	      tpos++;
+	      break;
+
+	    case STI:		/* I* implies I or nothing */
+	      if (!isgap(aseq[idx][mtr->pos[mpos]-1])) {
+		TraceSet(tr[idx], tpos, STI, mtr->nodeidx[mpos], i);
+		i++;
+		tpos++;
+	      }
+	      break;
+
+	    case STJ:		/* N,J,C: first N* -> N. After that, N* -> N or nothing. */
+	    case STN:
+	    case STC:
+	      if (mtr->pos[mpos] == 0) { 
+		TraceSet(tr[idx], tpos, mtr->statetype[mpos], 0, 0);
+		tpos++;
+	      } else if (!isgap(aseq[idx][mtr->pos[mpos]-1])) {
+		TraceSet(tr[idx], tpos, mtr->statetype[mpos], 0, i);
+		i++;
+		tpos++; 
+	      }
+	      break;
+
+	    case STBOGUS:
+	      Die("never happens. Trust me.");
+	    }
+	}
+      tr[idx]->tlen = tpos;
+    }	  
+  *ret_tr = tr;
+}
+
+
 /* Function: rightjustify()
  * 
  * Purpose:  Given a gap-containing string of length n,
@@ -981,3 +1188,5 @@ rightjustify(char *s, int n)
   while (npos >= 0) 
     s[npos--] = '.';
 }
+
+
