@@ -1,21 +1,21 @@
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
  * Copyright (C) 1992-1998 Washington University School of Medicine
- *
- *   This source code is distributed under the terms of the 
- *   GNU General Public License. See the files COPYING and 
- *   GNULICENSE for details.
- *    
+ * All Rights Reserved
+ * 
+ *     This source code is distributed under the terms of the
+ *     GNU General Public License. See the files COPYING and LICENSE
+ *     for details.
  ************************************************************/
 
 /* version.h
  * Version and copyright stamp information for the package.
- * RCS $Header: /nfs/wol2/people/eddy/src/hmmer/src/RCS/version.h,v 1.8 1998/05/28 13:37:26 eddy Exp $
+ * RCS $Header: /nfs/wol2/people/eddy/src/hmmer/src/RCS/version.h,v 1.12 1998/12/08 16:31:38 eddy Exp $
  */
 
 
-#define RELEASE     "2.0"
-#define RELEASEDATE "June 1998"
+#define RELEASE     "2.1.1"
+#define RELEASEDATE "Dec 1998"
 
 #ifndef LINTING			/* compiled-in stamp for all binaries  */
 static char *copyright = "\
@@ -23,5 +23,6 @@ static char *copyright = "\
    Copyright (C) 1992-1998 Washington University School of Medicine.\n\
 \n\
    HMMER is distributed under the terms of the GNU General Public\n\
-   License. See the files COPYING and GNULICENSE for details.";
+   License, unless otherwise stated; see the files COPYING and LICENSE\n\
+   for details.";
 #endif /* LINTING */
